package com.zanxiang.game.module.manage.controller;

import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.game.module.base.domain.ResultVO;
import com.zanxiang.game.module.manage.domain.params.GamePictureParam;
import com.zanxiang.game.module.manage.domain.vo.GamePictureVO;
import com.zanxiang.game.module.manage.service.IGamePictureService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author : lingfeng
 * @time : 2022-07-12
 * @description : 游戏图片
 */
@Api(tags = {"游戏图片管理接口"})
@RestController
@RequestMapping("/game/picture")
@Slf4j
public class GamePictureController {

    @Autowired
    private IGamePictureService gamePictureService;

    @ApiOperation(value = "获取游戏图片配置")
    @GetMapping(value = "/info")
    @PreAuthorize(permissionKey = "sdk:gamePicture:info")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = GamePictureVO.class)})
    public ResultVO<GamePictureVO> getByGameId(@RequestParam Long id) {
        return new ResultVO<>(gamePictureService.getByGameId(id));
    }

    @ApiOperation(value = "游戏图片配置添加或者更新")
    @PostMapping(value = "/add/or/update")
    @PreAuthorize(permissionKey = "sdk:gamePicture:addOrUpdate")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> addOrUpdate(@Validated @RequestBody GamePictureParam param) {
        return new ResultVO<>(gamePictureService.addOrUpdate(param));
    }

}
