package com.zanxiang.game.module.manage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.game.module.base.domain.ResultVO;
import com.zanxiang.game.module.manage.domain.params.PromoSiteAddOrUpdateParam;
import com.zanxiang.game.module.manage.domain.params.PromoSiteListParam;
import com.zanxiang.game.module.manage.domain.vo.PromoSiteListVO;
import com.zanxiang.game.module.manage.service.IPromoSiteService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author : lingfeng
 * @time : 2022-11-18
 * @description : 推广版位
 */
@Api(tags = "推广版位接口")
@RestController
@RequestMapping("/promo/site")
public class PromoSiteController {

    @Autowired
    private IPromoSiteService promoSiteService;

    @ApiOperation(value = "查询推广媒体版位列表")
    @PostMapping(value = "/list")
    @PreAuthorize(permissionKey = "manage:promoSite:list")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = PromoSiteListVO.class)})
    public ResultVO<IPage<PromoSiteListVO>> list(@Validated @RequestBody PromoSiteListParam param) {
        return ResultVO.ok(promoSiteService.list(param));
    }

    @ApiOperation(value = "推广媒体版位添加或修改")
    @PostMapping(value = "/add/or/update")
    @PreAuthorize(permissionKey = "manage:promoSite:addOrUpdate")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> addOrUpdate(@Validated @RequestBody PromoSiteAddOrUpdateParam param) {
        return ResultVO.ok(promoSiteService.addOrUpdate(param));
    }

    @ApiOperation(value = "推广媒体版位状态变更")
    @PatchMapping(value = "/status/update")
    @PreAuthorize(permissionKey = "manage:promoSite:statusUpdate")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> statusUpdate(@RequestParam Long id, @RequestParam Integer status) {
        return new ResultVO<>(promoSiteService.statusUpdate(id, status));
    }
}
