package com.zanxiang.game.module.manage.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.game.module.manage.domain.dto.GameTagDTO;
import com.zanxiang.game.module.manage.domain.params.GameTagAddUpdateParam;
import com.zanxiang.game.module.manage.domain.params.GameTagListParam;
import com.zanxiang.game.module.manage.domain.vo.GameTagChoiceVO;
import com.zanxiang.game.module.manage.domain.vo.GameTagVO;
import com.zanxiang.game.module.mybatis.entity.GameTag;

import java.util.List;
import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2022-11-16
 * @description : 游戏分类标签
 */
public interface IGameTagService extends IService<GameTag> {

    /**
     * 根据主键id查询分类以及子分类
     *
     * @param ids : 主键id列表
     * @return {@link List}<{@link GameTagVO}>
     */
    List<GameTagVO> listByIds(List<Long> ids);

    /**
     * 游戏标签选择列表
     *
     * @return {@link List}<{@link GameTagChoiceVO}>
     */
    List<GameTagChoiceVO> gameTagsChoiceList();

    /**
     * 查询游戏分类标签
     *
     * @return {@link Map}<{@link Long}, {@link GameTagDTO}>
     */
    Map<Long, GameTagDTO> gameTagMap();

    /**
     * 查询游戏标签列表
     *
     * @param param : 游戏标签列表查询呢参数
     * @return {@link IPage}<{@link GameTagVO}>
     */
    IPage<GameTagVO> list(GameTagListParam param);

    /**
     * 游戏标签分类添加或修改
     *
     * @param param : 参数
     * @return {@link Boolean}
     */
    Boolean addOrUpdate(GameTagAddUpdateParam param);

    /**
     * 根据主键id删除
     *
     * @param id : 主键id
     * @return {@link Boolean}
     */
    Boolean deleteById(Long id);
}
