package com.zanxiang.game.module.manage.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.game.module.manage.domain.dto.PromoChannelDTO;
import com.zanxiang.game.module.manage.domain.params.PromoChannelAddUpdateParam;
import com.zanxiang.game.module.manage.domain.params.PromoChannelListParam;
import com.zanxiang.game.module.manage.domain.vo.PromoChannelChoiceVO;
import com.zanxiang.game.module.manage.domain.vo.PromoChannelListVO;
import com.zanxiang.game.module.mybatis.entity.PromoChannel;

import java.util.List;
import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2022-10-31
 * @description : 推广渠道
 */
public interface IPromoChannelService extends IService<PromoChannel> {

    /**
     * 查询推广媒体列表
     *
     * @param param : 推广媒体列表查询参数
     * @return {@link IPage}<{@link PromoChannelListVO}>
     */
    IPage<PromoChannelListVO> list(PromoChannelListParam param);

    /**
     * 推广账号新增或者更新
     *
     * @param param : 更新参数
     * @return {@link Boolean}
     */
    Boolean addOrUpdate(PromoChannelAddUpdateParam param);

    /**
     * 根据主键更新状态
     *
     * @param id     : 主键id
     * @param status : 状态
     * @return {@link Boolean}
     */
    Boolean statusUpdate(Long id, Integer status);

    /**
     * 注册渠道选择列表查询
     *
     * @return {@link List}<{@link PromoChannelChoiceVO}>
     */
    List<PromoChannelChoiceVO> promoChannelChoiceList();

    /**
     * 注册渠道信息
     *
     * @return {@link Map}<{@link Long}, {@link PromoChannelDTO}>
     */
    Map<Long, PromoChannelDTO> promoChannelMap();

    /**
     * 注册渠道信息
     *
     * @param agentIdList 代理id列表
     * @return {@link Map}<{@link Long}, {@link PromoChannelDTO}>
     */
    Map<Long, PromoChannelDTO> promoChannelMap(List<Long> agentIdList);

    /**
     * 通过帐户或投手id列表
     *
     * @param account   账户
     * @param pitcherId 投手id
     * @return {@link List}<{@link Long}>
     */
    List<Long> listByAccountOrPitcherId(String account, String pitcherId);
}
