package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.module.base.enums.GameStatusEnum;
import com.zanxiang.game.module.base.utils.bean.BeanUtils;
import com.zanxiang.game.module.manage.domain.dto.GameCategoryDTO;
import com.zanxiang.game.module.manage.domain.dto.GameDTO;
import com.zanxiang.game.module.manage.domain.dto.GameTagDTO;
import com.zanxiang.game.module.manage.domain.params.*;
import com.zanxiang.game.module.manage.domain.vo.*;
import com.zanxiang.game.module.manage.service.*;
import com.zanxiang.game.module.mybatis.entity.*;
import com.zanxiang.game.module.mybatis.mapper.GameMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-06-24
 * @description : 游戏逻辑
 */
@Slf4j
@Service
public class GameServiceImpl extends ServiceImpl<GameMapper, Game> implements IGameService {

    @Autowired
    private IGamePayWayService gamePayWayService;

    @Autowired
    private IGameCategoryService gameCategoryService;

    @Autowired
    private IGameTagService gameTagService;

    @Autowired
    private ICpService cpService;

    @Autowired
    private IGameAppletService gameAppletService;

    @Override
    public GameDockVO getGameDock(Long id) {
        Game game = super.getById(id);
        GameDockVO gameDockVO = BeanUtils.copy(game, GameDockVO.class);
        if (gameDockVO == null) {
            return null;
        }
        Cp cp = cpService.getById(gameDockVO.getCpId());
        if (cp != null) {
            gameDockVO.setCpName(cp.getCpName());
        }
        GameApplet gameApplet = gameAppletService.getOne(new LambdaQueryWrapper<GameApplet>().eq(GameApplet::getGameId, gameDockVO.getId()));
        if (gameApplet != null) {
            gameDockVO.setAppId(gameApplet.getAppId());
            gameDockVO.setAppSecret(gameApplet.getAppSecret());
        }
        return gameDockVO;
    }

    @Override
    public Boolean gameDockUpdate(GameDockParam param) {
        if (Strings.isNotBlank(param.getAppKey())) {
            super.update(new LambdaUpdateWrapper<Game>()
                    .set(Game::getAppKey, param.getAppKey())
                    .set(Game::getUpdateTime, LocalDateTime.now())
                    .eq(Game::getId, param.getId()));
        }
        if (Strings.isBlank(param.getAppId()) && Strings.isBlank(param.getAppSecret())) {
            return Boolean.TRUE;
        }
        GameApplet gameApplet = gameAppletService.getOne(new LambdaQueryWrapper<GameApplet>()
                .eq(GameApplet::getGameId, param.getId()));
        if (gameApplet == null) {
            gameApplet = GameApplet.builder()
                    .gameId(param.getId())
                    .createTime(LocalDateTime.now())
                    .build();
        }
        gameApplet.setUpdateTime(LocalDateTime.now());
        if (Strings.isNotBlank(param.getAppId())) {
            gameApplet.setAppId(param.getAppId());
        }
        if (Strings.isNotBlank(param.getAppSecret())) {
            gameApplet.setAppSecret(param.getAppSecret());
        }
        return gameAppletService.saveOrUpdate(gameApplet);
    }

    @Override
    public GameRelationVO getGameRelation(Long id) {
        Game game = super.getById(id);
        GameRelationVO gameRelationVO = BeanUtils.copy(game, GameRelationVO.class);
        if (gameRelationVO == null) {
            return null;
        }
        Map<Long, Game> gameMap = this.getGameRelationMap(Collections.singletonList(game));
        //没有关联游戏
        if (gameMap.isEmpty()) {
            return gameRelationVO;
        }
        //设置关联游戏名字
        if (Objects.equals(gameRelationVO.getParentId(), 0L)) {
            gameRelationVO.setParentId(game.getId());
            gameRelationVO.setParentName(game.getName());
            gameRelationVO.setParentCategoryId(game.getCategory());
        } else {
            //非主游戏
            if (gameRelationVO.getParentId() != null && gameMap.containsKey(gameRelationVO.getParentId())) {
                gameRelationVO.setParentName(gameMap.get(gameRelationVO.getParentId()).getName());
                gameRelationVO.setParentCategoryId(gameMap.get(gameRelationVO.getParentId()).getCategory());
            }
        }
        if (gameRelationVO.getGuideGameId() != null && gameMap.containsKey(gameRelationVO.getGuideGameId())) {
            gameRelationVO.setGuideGameName(gameMap.get(gameRelationVO.getGuideGameId()).getName());
            gameRelationVO.setGuideCategoryId(gameMap.get(gameRelationVO.getGuideGameId()).getCategory());
        }
        if (gameRelationVO.getH5GameId() != null && gameMap.containsKey(gameRelationVO.getH5GameId())) {
            gameRelationVO.setH5GameName(gameMap.get(gameRelationVO.getH5GameId()).getName());
        }
        //设置关联游戏应用类型名字
        Set<Long> categoryIdSet = new HashSet<>();
        if (gameRelationVO.getParentCategoryId() != null) {
            categoryIdSet.add(gameRelationVO.getParentCategoryId());
        }
        if (gameRelationVO.getGuideCategoryId() != null) {
            categoryIdSet.add(gameRelationVO.getGuideCategoryId());
        }
        Map<Long, GameCategory> categoryMap = gameCategoryService.listByIds(categoryIdSet).stream()
                .collect(Collectors.toMap(GameCategory::getId, Function.identity()));
        if (gameRelationVO.getParentCategoryId() != null && categoryMap.containsKey(gameRelationVO.getParentCategoryId())) {
            gameRelationVO.setParentCategoryName(categoryMap.get(gameRelationVO.getParentCategoryId()).getName());
        }
        if (gameRelationVO.getGuideCategoryId() != null && categoryMap.containsKey(gameRelationVO.getGuideCategoryId())) {
            gameRelationVO.setGuideCategoryName(categoryMap.get(gameRelationVO.getGuideCategoryId()).getName());
        }
        return gameRelationVO;
    }

    private Map<Long, Game> getGameRelationMap(List<Game> gameList) {
        Set<Long> gameIdSet = new HashSet<>();
        gameList.forEach(game -> {
            if (game.getParentId() != null && !Objects.equals(game.getParentId(), 0L)) {
                gameIdSet.add(game.getParentId());
            }
            if (game.getH5GameId() != null) {
                gameIdSet.add(game.getH5GameId());
            }
            if (game.getGuideGameId() != null) {
                gameIdSet.add(game.getGuideGameId());
            }
        });
        if (gameIdSet.isEmpty()) {
            return Collections.emptyMap();
        }
        return super.listByIds(gameIdSet).stream().collect(Collectors.toMap(Game::getId, Function.identity()));
    }

    @Override
    public Boolean guideGameUpdateUpdate(GuideGameUpdateParam param) {
        return super.update(new LambdaUpdateWrapper<Game>()
                .set(param.getGuideGameId() != null, Game::getGuideGameId, param.getGuideGameId())
                .set(Game::getUpdateTime, LocalDateTime.now())
                .eq(Game::getId, param.getId()));
    }

    @Override
    public Boolean relationGameUpdate(RelationGameUpdateParam param) {
        if (param.getParentId() == null && param.getH5GameId() == null) {
            return Boolean.FALSE;
        }
        return super.update(new LambdaUpdateWrapper<Game>()
                .set(param.getParentId() != null, Game::getParentId, param.getParentId())
                .set(param.getH5GameId() != null, Game::getH5GameId, param.getH5GameId())
                .set(Game::getUpdateTime, LocalDateTime.now())
                .eq(Game::getId, param.getId()));
    }

    @Override
    public GameInfoVO getGameInfo(Long gameId) {
        Game game = super.getById(gameId);
        GameInfoVO gameInfoVO = BeanUtils.copy(game, GameInfoVO.class);
        if (gameInfoVO == null) {
            return null;
        }
        if (gameInfoVO.getCpId() != null) {
            Cp cp = cpService.getById(gameInfoVO.getCpId());
            if (cp != null) {
                gameInfoVO.setCpName(cp.getCpName());
            }
        }
        if (Strings.isNotBlank(game.getTags())) {
            String[] split = game.getTags().split(",");
            List<Long> idList = Arrays.stream(split).map(Long::valueOf).collect(Collectors.toList());
            List<GameTagVO> gameTagVOList = gameTagService.listByIds(idList);
            gameInfoVO.setClassifyList(BeanUtils.copyList(gameTagVOList, GameTagChoiceVO.class));
        }
        if (game.getCategory() != null) {
            GameCategoryDTO gameCategoryDTO = gameCategoryService.getById(game.getCategory());
            gameInfoVO.setCategory(gameCategoryDTO);
        }
        //是否主游戏
        gameInfoVO.setIsParentGame(Objects.equals(gameInfoVO.getParentId(), 0L));
        //关联游戏查询
        Map<Long, Game> gameMap = this.getGameRelationMap(Collections.singletonList(game));
        //没有关联游戏
        if (gameMap.isEmpty()) {
            return gameInfoVO;
        }
        //设置关联游戏名字
        if (gameInfoVO.getParentId() != null && !Objects.equals(gameInfoVO.getParentId(), 0L)) {
            gameInfoVO.setParentName(gameMap.get(gameInfoVO.getParentId()).getName());
        } else {
            gameInfoVO.setParentId(game.getId());
            gameInfoVO.setParentName(game.getName());
        }
        if (gameInfoVO.getH5GameId() != null) {
            gameInfoVO.setH5GameName(gameMap.get(gameInfoVO.getH5GameId()).getName());
        }
        if (gameInfoVO.getGuideGameId() != null) {
            gameInfoVO.setGuideGameName(gameMap.get(gameInfoVO.getGuideGameId()).getName());
        }
        return gameInfoVO;
    }

    @Override
    public Boolean updateGameInfo(GameUpdateParam param) {
        //游戏分类处理
        String gameTags = this.getGameTags(param.getClassifyList());
        return super.update(new LambdaUpdateWrapper<Game>()
                .set(Strings.isNotBlank(param.getName()), Game::getName, param.getName())
                .set(param.getCpId() != null, Game::getCpId, param.getCpId())
                .set(Strings.isNotBlank(gameTags), Game::getTags, gameTags)
                .set(param.getCategory() != null, Game::getCategory, param.getCategory())
                .set(param.getShareScale() != null, Game::getShareScale, param.getShareScale())
                .set(Objects.equals(param.getIsParentGame(), Boolean.TRUE), Game::getParentId, 0L)
                .set(!Objects.equals(param.getIsParentGame(), Boolean.TRUE) && param.getParentId() != null,
                        Game::getParentId, param.getParentId())
                .set(param.getH5GameId() != null, Game::getH5GameId, param.getH5GameId())
                .set(param.getGuideGameId() != null, Game::getGuideGameId, param.getGuideGameId())
                .set(Strings.isNotBlank(param.getPublicity()), Game::getPublicity, param.getPublicity())
                .set(Strings.isNotBlank(param.getDescription()), Game::getDescription, param.getDescription())
                .set(Strings.isNotBlank(param.getVersion()), Game::getVersion, param.getVersion())
                .set(Strings.isNotBlank(param.getGameUrl()), Game::getGameUrl, param.getGameUrl())
                .set(Strings.isNotBlank(param.getCpPaybackUrl()), Game::getCpPaybackUrl, param.getCpPaybackUrl())
                .set(Game::getUpdateTime, LocalDateTime.now())
                .eq(Game::getId, param.getId()));
    }

    @Override
    public Boolean gameAddOrUpdate(GameAddParam param) {
        //游戏分类处理
        String gameTags = this.getGameTags(param.getClassifyList());
        //主键id
        Long id = param.getId();
        Game game;
        if (id == null) {
            game = Game.builder()
                    .cpId(param.getCpId())
                    .name(param.getName())
                    .shareScale(param.getShareScale())
                    .category(param.getCategory())
                    .tags(gameTags)
                    .cpPaybackUrl(param.getCpPaybackUrl())
                    .gameUrl(param.getGameUrl())
                    .status(GameStatusEnum.UN_JOIN_UP.getStatus())
                    .createTime(LocalDateTime.now())
                    .updateTime(LocalDateTime.now())
                    .build();
        } else {
            game = super.getById(id);
            game.setCpId(param.getCpId());
            game.setName(param.getName());
            game.setShareScale(param.getShareScale());
            game.setTags(gameTags);
            game.setUpdateTime(LocalDateTime.now());
            game.setCpPaybackUrl(param.getCpPaybackUrl());
            game.setGameUrl(param.getGameUrl());
        }
        //设置父游戏id
        if (!Objects.equals(param.getIsParentGame(), Boolean.TRUE) && param.getParentGameId() != null) {
            game.setParentId(param.getParentGameId());
        }
        return super.saveOrUpdate(game);
    }

    private String getGameTags(List<Long> tagIdList) {
        if (CollectionUtils.isEmpty(tagIdList)) {
            return null;
        }
        List<GameTagVO> gameTagVOList = gameTagService.listByIds(tagIdList);
        Set<Long> tagIdSet = new HashSet<>();
        gameTagVOList.forEach(tag -> {
            tagIdSet.add(tag.getId());
            if (Objects.equals(tag.getIsParent(), Boolean.TRUE) && tag.getParentId() != null) {
                tagIdSet.add(tag.getParentId());
            }
        });
        if (tagIdSet.isEmpty()) {
            return null;
        }
        return StringUtils.join(tagIdSet, ",");
    }

    @Override
    public IPage<GameListVO> gameList(GameListParam param) {
        //cp列表
        Map<Long, String> cpMap = cpService.choiceList().stream()
                .collect(Collectors.toMap(CpChoiceVO::getId, CpChoiceVO::getCpName));
        //游戏应用类型列表
        Map<Long, GameCategoryDTO> gameCategoryMap = gameCategoryService.gameCategoryMap();
        //游戏分类标签列表
        Map<Long, GameTagDTO> gameTagMap = gameTagService.gameTagMap();
        //查询游戏
        Map<Long, String> gameMap = this.choiceList().stream()
                .collect(Collectors.toMap(GameChoiceVO::getId, GameChoiceVO::getName));
        //分类标签
        List<Long> tagIds = new ArrayList<>();
        if (param.getGameClassifyId() != null) {
            if (Objects.equals(param.getIsParentClassify(), Boolean.TRUE)) {
                tagIds = gameTagService.list(new LambdaQueryWrapper<GameTag>()
                        .eq(GameTag::getParentId, param.getGameClassifyId()))
                        .stream().map(GameTag::getId).collect(Collectors.toList());
            }
        }
        final List<Long> tagIdList = tagIds;
        //执行查询
        return page(param.toPage(), new QueryWrapper<Game>().lambda()
                .eq(param.getCpId() != null, Game::getCpId, param.getCpId())
                .eq(param.getGameId() != null && !Objects.equals(param.getIsParentGame(), Boolean.TRUE), Game::getId, param.getGameId())
                .and(param.getGameId() != null && Objects.equals(param.getIsParentGame(), Boolean.TRUE),
                        qw -> qw.eq(Game::getId, param.getGameId()).or().eq(Game::getParentId, param.getGameId()))
                .eq(param.getGameCategoryId() != null, Game::getCategory, param.getGameCategoryId())
                .eq(param.getH5GameId() != null, Game::getH5GameId, param.getH5GameId())
                .eq(param.getGuideGameId() != null, Game::getGuideGameId, param.getGuideGameId())
                .eq(param.getStatus() != null, Game::getStatus, param.getStatus())
                //非父标签查询
                .apply(param.getGameClassifyId() != null && !Objects.equals(param.getIsParentClassify(), Boolean.TRUE),
                        "FIND_IN_SET({0}, tags)", String.valueOf(param.getGameClassifyId()))
                //父标签查询
                .and(param.getGameClassifyId() != null && Objects.equals(param.getIsParentClassify(), Boolean.TRUE),
                        qw -> {
                            qw.apply("FIND_IN_SET({0}, tags)", String.valueOf(param.getGameClassifyId()));
                            for (Long tagId : tagIdList) {
                                qw.or().apply("FIND_IN_SET({0}, tags)", String.valueOf(tagId));
                            }
                        }
                )
                .orderByDesc(Game::getCreateTime)
        ).convert(game -> this.toVo(game, cpMap, gameCategoryMap, gameMap, gameTagMap));
    }

    private GameListVO toVo(Game game, Map<Long, String> cpMap, Map<Long, GameCategoryDTO> gameCategoryMap,
                            Map<Long, String> gameMap, Map<Long, GameTagDTO> gameTagMap) {
        GameListVO gameListVO = BeanUtils.copy(game, GameListVO.class);
        if (gameListVO == null) {
            return null;
        }
        gameListVO.setClassify(game.getTags());
        //cp设置
        if (gameListVO.getCpId() != null) {
            gameListVO.setCpName(cpMap.get(gameListVO.getCpId()));
        }
        //游戏类型
        if (gameListVO.getCategory() != null) {
            gameListVO.setCategoryName(gameCategoryMap.get(gameListVO.getCategory()) == null ? null : gameCategoryMap.get(gameListVO.getCategory()).getName());
        }
        //游戏分类
        if (gameListVO.getClassify() != null) {
            String[] classifyIds = gameListVO.getClassify().split(",");
            List<String> tagNameList = new ArrayList<>();
            for (String tagId : classifyIds) {
                tagNameList.add(gameTagMap.get(Long.valueOf(tagId)) == null ? null : gameTagMap.get(Long.valueOf(tagId)).getName());
            }
            gameListVO.setClassifyNameList(tagNameList);
            gameListVO.setTagNameList(tagNameList);
        }
        //主游戏设置
        if (Objects.equals(game.getParentId(), 0L)) {
            gameListVO.setIsParentGame(Boolean.TRUE);
            gameListVO.setParentId(game.getId());
            gameListVO.setParentName(game.getName());
        } else {
            gameListVO.setIsParentGame(Boolean.FALSE);
            gameListVO.setParentName(gameMap.get(gameListVO.getParentId()));
        }
        //h5游戏
        if (gameListVO.getH5GameId() != null) {
            gameListVO.setH5GameName(gameMap.get(gameListVO.getH5GameId()));
        }
        //关联游戏
        if (gameListVO.getGuideGameId() != null) {
            gameListVO.setGuideGameName(gameMap.get(gameListVO.getGuideGameId()));
        }
        //查询支付方式列表
        List<GamePayWayVO> gamePayWayVOList = gamePayWayService.getByGameId(game.getId());
        gameListVO.setGamePayWayList(gamePayWayVOList);
        return gameListVO;
    }

    @Override
    public List<GameChoiceVO> choiceList() {
        List<Game> gameList = super.list(new LambdaQueryWrapper<Game>().select(Game::getId, Game::getName, Game::getCategory));
        return BeanUtils.copyList(gameList, GameChoiceVO.class);
    }

    @Override
    public GameDTO getById(Long id) {
        if (id == null) {
            return null;
        }
        return BeanUtils.copy(super.getById(id), GameDTO.class);
    }

    @Override
    public Map<Long, GameDTO> gameCondition(Long cpId, Long gameId, Long gameCategoryId) {
        //游戏条件判断
        if (cpId == null && gameId == null && gameCategoryId == null) {
            return null;
        }
        List<GameDTO> gameList = this.getGameList(cpId, gameId, gameCategoryId);
        if (CollectionUtils.isEmpty(gameList)) {
            return Collections.emptyMap();
        }
        return gameList.stream().collect(Collectors.toMap(GameDTO::getId, Function.identity()));
    }

    private List<GameDTO> getGameList(Long cpId, Long gameId, Long gameCategoryId) {
        if (cpId == null && gameId == null && gameCategoryId == null) {
            return Collections.emptyList();
        }
        List<Game> gameList = super.list(new LambdaQueryWrapper<Game>()
                .eq(cpId != null, Game::getCpId, cpId)
                .eq(gameId != null, Game::getId, gameId)
                .eq(gameCategoryId != null, Game::getCategory, gameCategoryId)
                .select(Game::getId, Game::getCpId, Game::getName, Game::getCategory)
        );
        return BeanUtils.copyList(gameList, GameDTO.class);
    }

    @Override
    public Map<Long, GameDTO> gameMap(Collection<Long> gameIds) {
        if (CollectionUtils.isEmpty(gameIds)) {
            return Collections.emptyMap();
        }
        List<Game> gameList = super.listByIds(gameIds);
        if (CollectionUtils.isEmpty(gameList)) {
            return Collections.emptyMap();
        }
        List<GameDTO> gameDTOList = BeanUtils.copyList(gameList, GameDTO.class);
        return gameDTOList.stream().collect(Collectors.toMap(GameDTO::getId, Function.identity()));
    }

    @Override
    public Map<Long, GameDTO> gameMap() {
        List<Game> gameList = super.list();
        if (CollectionUtils.isEmpty(gameList)) {
            return Collections.emptyMap();
        }
        List<GameDTO> gameDTOList = BeanUtils.copyList(gameList, GameDTO.class);
        return gameDTOList.stream().collect(Collectors.toMap(GameDTO::getId, Function.identity()));
    }

    @Override
    public Boolean deleteById(Long id) {
        return super.removeById(id);
    }

    @Override
    public Boolean statusUpdate(Long id, Integer status) {
        return super.update(new LambdaUpdateWrapper<Game>()
                .set(Game::getStatus, status)
                .set(Game::getUpdateTime, LocalDateTime.now())
                .eq(Game::getId, id));
    }

}
