package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.module.base.utils.StringUtils;
import com.zanxiang.game.module.base.utils.bean.BeanUtils;
import com.zanxiang.game.module.manage.domain.dto.*;
import com.zanxiang.game.module.manage.domain.params.GameUserRoleListParam;
import com.zanxiang.game.module.manage.domain.vo.GameUserRoleListVO;
import com.zanxiang.game.module.manage.domain.vo.GameUserRoleVO;
import com.zanxiang.game.module.manage.domain.vo.GameUserVO;
import com.zanxiang.game.module.manage.service.*;
import com.zanxiang.game.module.mybatis.entity.GameUserRole;
import com.zanxiang.game.module.mybatis.entity.PromoChannel;
import com.zanxiang.game.module.mybatis.mapper.GameUserRoleMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;

/**
 * @author : lingfeng
 * @time : 2022-07-01
 * @description : 玩家角色
 */
@Slf4j
@Service
public class GameUserRoleServiceImpl extends ServiceImpl<GameUserRoleMapper, GameUserRole> implements IGameUserRoleService {

    @Autowired
    private IUserService userService;

    @Autowired
    private IGameService gameService;

    @Autowired
    private IGameUserService gameUserService;

    @Autowired
    private IGameCategoryService gameCategoryService;

    @Autowired
    private ICpService cpService;

    @Autowired
    private IPromoChannelService promoChannelService;

    @Override
    public GameUserRoleDTO getByRoleId(String roleId) {
        GameUserRole gameUserRole = super.getOne(new LambdaQueryWrapper<GameUserRole>().eq(GameUserRole::getRoleId, roleId));
        return BeanUtils.copy(gameUserRole, GameUserRoleDTO.class);
    }

    @Override
    public List<GameUserRoleVO> roleList(Long userId) {
        List<GameUserRole> gameUserRoleList = super.list(new LambdaQueryWrapper<GameUserRole>().eq(GameUserRole::getUserId, userId));
        if (CollectionUtils.isEmpty(gameUserRoleList)) {
            return Collections.emptyList();
        }
        List<GameUserRoleVO> list = new ArrayList<>();
        gameUserRoleList.forEach(role -> {
            GameUserRoleVO gameUserRoleVO = BeanUtils.copy(role, GameUserRoleVO.class);
            if (gameUserRoleVO == null) {
                return;
            }
            //设置游戏名称
            GameDTO gameDTO = gameService.getById(role.getGameId());
            if (gameDTO != null) {
                gameUserRoleVO.setGameName(gameDTO.getName());
                //设置游戏列表
                GameCategoryDTO gameCategoryDTO = gameCategoryService.getById(gameDTO.getCategory());
                if (gameCategoryDTO != null) {
                    gameUserRoleVO.setGameCategoryName(gameCategoryDTO.getName());
                }
            }
            //玩家游戏信息
            GameUserVO gameUserVO = gameUserService.getById(role.getGameUserId());
            if (gameUserVO != null) {
                PromoChannel promoChannel = promoChannelService.getById(gameUserVO.getAgentId());
                if (promoChannel != null) {
                    gameUserRoleVO.setChannelName(promoChannel.getChannel());
                }
            }
            list.add(gameUserRoleVO);
        });
        return list;
    }

    @Override
    public IPage<GameUserRoleListVO> list(GameUserRoleListParam param) {
        //玩家条件处理
        Map<Long, UserDTO> userMap = userService.userCondition(param.getUserId(), param.getChannelId() == null ? null : Collections.singletonList(param.getChannelId()),
                param.getUserName(), param.getNickname(), param.getRegIp(), null);
        //根据条件, 匹配不到玩家
        if (userMap != null && userMap.isEmpty()) {
            return new Page<>();
        }
        //相关用户id
        Set<Long> userIdSet = userMap == null ? null : userMap.keySet();
        //游戏条件处理
        Map<Long, GameDTO> gameMap = gameService.gameCondition(param.getCpId(), param.getGameId(), param.getGameCategoryId());
        //根据条件, 匹配不到游戏
        if (gameMap != null && gameMap.isEmpty()) {
            return new Page<>();
        }
        Map<Long, PromoChannelDTO> channelMap = promoChannelService.promoChannelMap();
        Map<Long, CpDTO> cpMap = cpService.cpMap();
        Map<Long, GameCategoryDTO> gameCategoryMap = gameCategoryService.gameCategoryMap();
        //执行查询
        return page(param.toPage(), new QueryWrapper<GameUserRole>().lambda()
                .in(CollectionUtils.isNotEmpty(userIdSet), GameUserRole::getUserId, userIdSet)
                .eq(Strings.isNotBlank(param.getServerName()), GameUserRole::getServerName, param.getServerName())
                .eq(Strings.isNotBlank(param.getRoleName()), GameUserRole::getRoleName, param.getRoleName())
                .in(gameMap != null, GameUserRole::getGameId, gameMap != null ? gameMap.keySet() : null)
                .eq(Strings.isNotBlank(param.getOs()), GameUserRole::getOs, param.getOs())
                .gt(Objects.equals(param.getIsRecharge(), Boolean.TRUE), GameUserRole::getRechargeCount, 0)
                .eq(Objects.equals(param.getIsRecharge(), Boolean.FALSE), GameUserRole::getRechargeCount, 0)
                .eq(param.getVipLevel() != null && param.getVipLevel() <= 10, GameUserRole::getRoleVipLevel, param.getVipLevel())
                .gt(param.getVipLevel() != null && param.getVipLevel() > 10, GameUserRole::getRoleVipLevel, param.getVipLevel())
                .ge(param.getBeginDate() != null, GameUserRole::getCreateTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, GameUserRole::getCreateTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .ge(param.getRechargeBeginDate() != null, GameUserRole::getLastRechargeTime, param.getRechargeBeginDate() == null ? null : LocalDateTime.of(param.getRechargeBeginDate(), LocalTime.MIN))
                .le(param.getRechargeEndDate() != null, GameUserRole::getLastRechargeTime, param.getRechargeEndDate() == null ? null : LocalDateTime.of(param.getRechargeEndDate(), LocalTime.MAX))
                .orderByDesc(GameUserRole::getCreateTime)
        ).convert(u -> this.toVo(u, userMap, gameMap, channelMap, cpMap, gameCategoryMap));
    }

    private GameUserRoleListVO toVo(GameUserRole gameUserRole, Map<Long, UserDTO> userMap, Map<Long, GameDTO> gameMap,
                                    Map<Long, PromoChannelDTO> channelMap, Map<Long, CpDTO> cpMap, Map<Long, GameCategoryDTO> categoryMap) {
        GameUserRoleListVO gameUserRoleListVO = BeanUtils.copy(gameUserRole, GameUserRoleListVO.class);
        if (Objects.isNull(gameUserRoleListVO)) {
            return null;
        }
        UserDTO userDTO = StringUtils.isEmpty(userMap) ? null : userMap.get(gameUserRoleListVO.getUserId());
        if (userDTO == null) {
            userDTO = userService.getById(gameUserRoleListVO.getUserId());
        }
        if (userDTO != null) {
            gameUserRoleListVO.setUsername(userDTO.getUsername());
            gameUserRoleListVO.setNickname(userDTO.getNickname());
            gameUserRoleListVO.setAgentId(userDTO.getAgentId());
            gameUserRoleListVO.setChannel(channelMap.get(userDTO.getAgentId()) == null ? null : channelMap.get(userDTO.getAgentId()).getChannel());
            gameUserRoleListVO.setRegIp(userDTO.getIp());
            gameUserRoleListVO.setRegTime(userDTO.getCreateTime());
        }
        GameDTO gameDTO = StringUtils.isEmpty(gameMap) ? null : gameMap.get(gameUserRoleListVO.getGameId());
        if (gameDTO == null) {
            gameDTO = gameService.getById(gameUserRoleListVO.getGameId());
        }
        if (gameDTO != null) {
            gameUserRoleListVO.setGameName(gameDTO.getName());
            gameUserRoleListVO.setCpId(gameDTO.getCpId());
            gameUserRoleListVO.setCpName(cpMap.get(gameDTO.getCpId()) == null ? null : cpMap.get(gameDTO.getCpId()).getCpName());
            gameUserRoleListVO.setGameCategoryId(gameDTO.getCategory());
            gameUserRoleListVO.setGameCategoryName(categoryMap.get(gameDTO.getCategory()) == null ? null : categoryMap.get(gameDTO.getCategory()).getName());
        }
        return gameUserRoleListVO;
    }
}
