package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.module.manage.domain.params.UserNameAuthListParam;
import com.zanxiang.game.module.manage.domain.vo.UserCardVO;
import com.zanxiang.game.module.manage.service.IUserCardService;
import com.zanxiang.game.module.mybatis.entity.UserCard;
import com.zanxiang.game.module.mybatis.mapper.UserCardMapper;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-09-26
 * @description : 用户实名认证
 */
@Slf4j
@Service
public class UserCardServiceImpl extends ServiceImpl<UserCardMapper, UserCard> implements IUserCardService {

    @Override
    public IPage<UserCardVO> getUserCardList(UserNameAuthListParam param) {
        return page(param.toPage(), new QueryWrapper<UserCard>().lambda()
                .eq(param.getUserId() != null, UserCard::getUserId, param.getUserId())
                .eq(Strings.isNotBlank(param.getUserName()), UserCard::getUsername, param.getUserName())
                .eq(Strings.isNotBlank(param.getNickname()), UserCard::getNickname, param.getNickname())
                .ge(param.getBeginDate() != null, UserCard::getRegTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, UserCard::getRegTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .eq(Strings.isNotBlank(param.getRealName()), UserCard::getCardName, param.getRealName())
                .orderByDesc(UserCard::getCreateTime))
                .convert(this::toVo);
    }

    private UserCardVO toVo(UserCard userCard) {
        if (Objects.isNull(userCard)) {
            return null;
        }
        UserCardVO userCardVO = BeanUtil.copy(userCard, UserCardVO.class);
        userCardVO.setCardId(userCard.getShowCardId());
        userCardVO.setCardName(userCard.getShowCardName());
        return userCardVO;
    }

    @Override
    public UserCardVO getByUserId(Long userId) {
        UserCard userCard = this.getOne(new LambdaQueryWrapper<UserCard>()
                .eq(UserCard::getUserId, userId));
        return this.toVo(userCard);
    }

}
